package org.signwriting;

import com.dtai.gwt.GadgetGraphics;

public class Bitmap {
    private byte[] bytes;
    private int width = 0;
    private int height = 0;
    private boolean valid = true;

    public Bitmap() {
        bytes = new byte[8];
    }

    public Bitmap(int width, int height) {
        this.width = width;
        this.height = height;
        bytes = new byte[(((height-1)*width + width-1)/8)+1];
    }

    public void setBit(int x, int y) {
        setBit(x, y, true);
    }

    public void clearBit(int x, int y) {
        setBit(x, y, false);
    }

    public void setBit(int x, int y, boolean val) {
        byte[] oldbytes = bytes;
        int oldwidth = width;
        int oldheight = height;
        width = Math.max(width,x+1);
        height = Math.max(height,y+1);
        int maxbyte = (((height-1)*width) + (width-1))/8;
        if (maxbyte >= bytes.length) {
            bytes = new byte[(maxbyte+1)*2];
            if (width == oldwidth) {
                System.arraycopy(oldbytes,0,bytes,0,oldbytes.length);
            }
        }
        int idx = y*width + x;
        int byteidx = idx/8;
        int bitidx = idx%8;
        if (width > oldwidth) {
            for (int row = height-1; row >= 0; row--) {
                int col;
                for (col = width-1; col >= oldwidth; col--) {
                    clearBit(col,row);
                }
                for (; col >= 0; col--) {
                    if (row < oldheight) {
                        setBit(col,row,getBit(col,row,oldbytes,oldwidth,oldheight));
                    } else {
                        clearBit(col,row);
                    }
                }
            }
        }
        if (val) {
            bytes[byteidx] |= 1<<bitidx;
        } else {
            bytes[byteidx] &= ~(1<<bitidx);
        }
    }

    public final boolean getBit(int x, int y) {
        return getBit(x, y, bytes, width, height);
    }

    private final boolean getBit(int x, int y, byte[] bytes, int width, int height) {
        int idx = y*width + x;
        int byteidx = idx/8;
        int bitidx = idx%8;
        return ((bytes[byteidx] & 1<<bitidx) != 0);
    }

    public final int getWidth() {
        return width;
    }

    public final int getHeight() {
        return height;
    }
    
    public void paint(GadgetGraphics g, int x, int y) {
        for (int i = 0; i < height; i++) {
            for (int j = 0; j < width; j++) {
                if (getBit(j,i)) {
                    int drawx = x+j;
                    int drawy = y+i;
                    g.drawLine(drawx, drawy, drawx, drawy);
                }
            }
        }
    }
    
    public void setValid(boolean valid) {
        this.valid = valid;
    }
    
    public boolean isValid() {
        return valid;
    }
}
