/****************************************************************
 **  Copyright 1996 by DTAI Incorporated, All Rights Reserved  **
 ****************************************************************
 **
 **  $Id: ColorCombo.java,v 1.1 1998/03/22 20:06:35 kadel Exp $
 **
 **  $Source: /cvs/classes/signwriting/ColorCombo.java,v $
 **
 ****************************************************************/

package org.signwriting;

import com.dtai.gwt.ComboBoxGadget;
import com.dtai.gwt.Gadget;
import java.awt.Color;
import java.util.Enumeration;
import java.util.Hashtable;

public class ColorCombo extends ComboBoxGadget {

    private Hashtable colors = new Hashtable();

    public ColorCombo() {
        super(8);
        initColors();
        fillList();
        colors.put( "Black", Color.black );
        colors.put( "White", Color.white );
    }
    
    public ColorCombo(Color color) {
        this();
        setColor(color);
    }

    private void initColors() {
        colors.put( "Gray", Color.gray );
        colors.put( "Dark Gray", Color.darkGray );
        colors.put( "Light Gray", Color.lightGray );
        addColorVariants( "Blue", Color.blue );
        addColorVariants( "Cyan", Color.cyan );
        addColorVariants( "Green", Color.green );
        addColorVariants( "Magenta", Color.magenta );
        addColorVariants( "Pink", Color.pink );
        addColorVariants( "Red", Color.red );
        addColorVariants( "Yellow", Color.yellow );
    }
    
    private void addColorVariants(String name, Color color) {
        colors.put(name, color);
        Color darker = color.darker();
        if (!darker.equals(color)) {
            colors.put("Dark "+name, darker);
        }
        Color brighter = color.brighter();
        if (!brighter.equals(color)) {
            colors.put("Light "+name, brighter);
        }
    }

    private void fillList() {
        add("Black");
        add("White");
        for ( Enumeration e = colors.keys(); e.hasMoreElements(); ) {
            String name = (String)e.nextElement();
            add(name);
        }
    }

    public Color getColor() {
        String name = getText().trim();
        if ( name.length() > 0 ) {
            if ( Character.isLowerCase(name.charAt(0)) ) {
                name = Character.toUpperCase(name.charAt(0))+name.substring(1);
            }
            int space = name.indexOf(' ');
            if ( space > 0 ) {
                if ( Character.isLowerCase(name.charAt(space+1)) ) {
                    name = name.substring(0,space+1)+
                           Character.toUpperCase(name.charAt(space+1))+
                           name.substring(space+2);
                }
            }
        }
        Color color = (Color)colors.get(name);
        if ( color != null ) {
            return color;
        }
        name = name.trim();
        int offset=0;
        if ( name.length() > 0 ) {
            if ( name.charAt( 0 ) == '#' ) {
                offset=1;
            }
            try {
                int r = Integer.valueOf( name.substring( offset, offset+2 ), 16 ).intValue();
                int g = Integer.valueOf( name.substring( offset+2, offset+4 ), 16 ).intValue();
                int b = Integer.valueOf( name.substring( offset+4, offset+6 ), 16 ).intValue();
                return new Color( r, g, b );
            }
            catch ( Exception e ) {
            }
        }
        return Gadget.defaultColor;
    }

    public void setColor( Color newColor ) {
        if (newColor == Gadget.transparent) {
            select("Transparent");
            return;
        }
        for ( Enumeration e = colors.keys(); e.hasMoreElements(); ) {
            String name = (String)e.nextElement();
            Color color = (Color)colors.get(name);
            if ( ( color == newColor ) ||
                 ( ( color != null ) &&
                   ( newColor != null ) &&
                   newColor.equals(color) ) ) {
                select(name);
                return;
            }
        }
        String r = Integer.toString( newColor.getRed(), 16 );
        if (r.length() == 1) {
            r = "0"+r;
        }
        String g = Integer.toString( newColor.getGreen(), 16 );
        if (g.length() == 1) {
            g = "0"+g;
        }
        String b = Integer.toString( newColor.getBlue(), 16 );
        if (b.length() == 1) {
            b = "0"+b;
        }
        setText( "#"+r+g+b);
    }
}
