package org.signwriting;

import com.dtai.gwt.Gadget;
import com.dtai.gwt.MenuBarGadget;
import com.dtai.gwt.MenuGadget;
import com.dtai.gwt.MenuItemGadget;
import com.dtai.gwt.SeparatorGadget;
import java11.awt.event.ActionEvent;
import java11.awt.event.ActionListener;
import java.util.MissingResourceException;

public class EditorMenuBar extends MenuBarGadget {

    private static String menubarPrefix = "EditorMenuBar.";
    
    public EditorMenuBar(SignWriter signwriter) {
        createFileMenu(signwriter);
        createEditMenu(signwriter);
        createToolsMenu(signwriter);
    }
    
    private void createFileMenu(final SignWriter signwriter) {
        String menuName = "File";
        String menuResourceKey = menubarPrefix+Resources.labelToKey(menuName);
        MenuGadget menu = addMenu(menuName, menuResourceKey);
        addItem(
            menu, menuResourceKey, "New",
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    signwriter.newDocument();
                }
            }
        );
        addItem(
            menu, menuResourceKey, "Open",
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    signwriter.openDocument();
                }
            }
        );
        menu.addSeparator();
        addItem(
            menu, menuResourceKey, "Save",
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    signwriter.saveDocument();
                }
            }
        );
        addItem(
            menu, menuResourceKey, "Save As",
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    signwriter.saveAsDocument();
                }
            }
        );
        menu.addSeparator();
        addItem(
            menu, menuResourceKey, "Print",
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    signwriter.printDocument();
                }
            }
        );
        menu.addSeparator();
        addItem(
            menu, menuResourceKey, "Exit",
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    signwriter.exit();
                }
            }
        );
    }
    
    private void createEditMenu(final SignWriter signwriter) {
        String menuName = "Edit";
        String menuResourceKey = menubarPrefix+Resources.labelToKey(menuName);
        MenuGadget menu = addMenu(menuName, menuResourceKey);
        addItem(
            menu, menuResourceKey, "Undo",
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    signwriter.undo();
                }
            }
        );
        menu.addSeparator();
        addItem(
            menu, menuResourceKey, "Cut",
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    signwriter.cut();
                }
            }
        );
        addItem(
            menu, menuResourceKey, "Copy",
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    signwriter.copy();
                }
            }
        );
        addItem(
            menu, menuResourceKey, "Paste",
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    signwriter.paste();
                }
            }
        );
        addItem(
            menu, menuResourceKey, "Delete",
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    signwriter.delete();
                }
            }
        );
        menu.addSeparator();
        addItem(
            menu, menuResourceKey, "Select All",
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    signwriter.selectAll();
                }
            }
        );
    }
    
    private void createToolsMenu(final SignWriter signwriter) {
        String menuName = "Tools";
        String menuResourceKey = menubarPrefix+Resources.labelToKey(menuName);
        MenuGadget menu = addMenu(menuName, menuResourceKey);
        addItem(
            menu, menuResourceKey, "Dictionary...",
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    signwriter.showDictionary();
                }
            }
        );
        addItem(
            menu, menuResourceKey, "Preferences...",
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    signwriter.editViewPreferences();
                }
            }
        );
        menu.addSeparator();
        addItem(
            menu, menuResourceKey, "About SignWriter...",
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    signwriter.showAboutDialog();
                }
            }
        );
    }

    private MenuGadget addMenu(String menuName, String resourceKey) {
        MenuGadget menu = new MenuGadget();
        Resources.apply(resourceKey, menuName, menu);
        add(menu);
        return menu;
    }

    private MenuItemGadget addItem(
        MenuGadget menu, String menuResourceKey,
        String itemName, ActionListener listener) {
        
        MenuItemGadget item = new MenuItemGadget();
        String resourceKey = menuResourceKey+"."+Resources.labelToKey(itemName);
        Resources.apply(resourceKey, itemName, item);
        item.addActionListener(listener);
        menu.add(item);
        return item;
    }
}
