package org.signwriting;

import com.dtai.gwt.GadgetGraphics;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Point;
import java.awt.Color;
import java.awt.Font;
import java.awt.Image;
import java.util.Vector;

public class Sign extends DocumentPart implements Cloneable, java.io.Serializable {
    
    static final long serialVersionUID = 5449532988507948360L;

    private SignComponent[] components = new SignComponent[1];
    private int componentCount = 0;
    private int nextSignPos = 0;
    private boolean word = true;
    
    transient private Vector images;
    transient private char fontIndex;
    transient Color selectedColor;
    transient boolean imageNotOK;
        // double negative so object initialization will work

    public Sign() {
    }
    
    public void setWord(boolean word) {
        this.word = word;
    }
    
    public boolean isWord() {
        return word;
    }
    
    private final Vector getImages() {
        if (images == null) {
            images = new Vector();
        }
        return images;
    }
    
    final boolean isImageOK() {
        return !imageNotOK;
    }
    
    final void setImageOK(boolean imageOK) {
        imageNotOK = !imageOK;
    }
    
    final char getFontIndex() {
        return fontIndex;
    }
    
    final void setFontIndex(char fontIndex) {
        this.fontIndex = fontIndex;
    }
    
    private int findImage(boolean thumbNail, Color color) {
        Vector images = getImages();
        int size = images.size();
        for (int i = 0; i < size; i++) {
            SignImageEntry entry = (SignImageEntry)images.elementAt(i);
            if (entry.matches(thumbNail,color)) {
                return i;
            }
        }
        return -1;
    }
    
    public Image getImage(boolean thumbNail, Color color) {
        Vector images = getImages();
        int index = findImage(thumbNail, color);
        if (index < 0) {
            return null;
        }
        return ((SignImageEntry)images.elementAt(index)).getImage();
    }
    
    public void addImage(Image image, boolean thumbNail, Color color) {
        synchronized(this) {
            Vector images = getImages();
            if (findImage(thumbNail, color) < 0) {
                images.addElement(
                    new SignImageEntry(image, this, thumbNail, color));
            }
        }
    }

    public void removeImageEntry(SignImageEntry entry) {
        synchronized(this) {
            Vector images = getImages();
            if (images != null) {
                int index = findImage(entry.isThumbNail(), entry.getColor());
                if (index >= 0) {
                    images.removeElementAt(index);
                }
            }
        }
    }
    
    public void finalize() {
        removeAllImages();
    }
    
    public void removeAllImages() {
        synchronized(this) {
            Vector images = getImages();
            this.images = null;
            int size = images.size();
            for (int i = 0; i < size; i++) {
                SignImageEntry entry = (SignImageEntry)images.elementAt(i);
                entry.dropFromCache();
            }
        }
    }

    public Object clone() {
        synchronized(this) {
            Sign newSign = new Sign();
            newSign.componentCount = componentCount;
            newSign.nextSignPos = nextSignPos;
            newSign.fontIndex = fontIndex;
            newSign.word = word;
            if (newSign.components.length != components.length) {
                newSign.components = new SignComponent[components.length];
            }
            for (int i = 0; i < componentCount; i++) {
                newSign.components[i] = (SignComponent)components[i].clone();
            }
            return newSign;
        }
    }

    public int hashCode() {
        int code = 0;
        for (int i = 0; i < componentCount; i++) {
            code += components[i].hashCode();
        }
        return code;
    }

    public boolean equals(Object obj) {
        if (obj.getClass() != getClass()) {
            return false;
        }
        Sign rhs = (Sign)obj;
        if (rhs.componentCount != componentCount) {
            return false;
        }
        for (int i = 0; i < componentCount; i++) {
            if (!rhs.components[i].equals(components[i])) {
                return false;
            }
        }
        return true;
    }

    private Color getSelectedColor() {
        if (selectedColor == null) {
            selectedColor = Color.red;
        }
        return selectedColor;
    }
    
    public final void setSelectedColor(Color selectedColor) {
        this.selectedColor = selectedColor;
    }

    public int getComponentCount() {
        return componentCount;
    }

    public SignComponent getComponent(int i) {
        return components[i];
    }

    public SignComponent[] getComponents() {
        SignComponent[] copy = new SignComponent[componentCount];
        for (int i = 0; i < componentCount; i++) {
            copy[i] = components[i];
        }
        return copy;
    }

    public void setNextSignPos(int nextSignPos) {
        this.nextSignPos = nextSignPos;
    }

    public void add(SignComponent component) {
	    if (componentCount >= components.length) {
	        SignComponent[] newarray = new SignComponent[components.length*2];
	        System.arraycopy(components,0,newarray,0,components.length);
	        components = newarray;
	    }
	    components[componentCount++] = component;
    }
    
    public int getComponentIndex(SignComponent component) {
        for (int i = 0; i < componentCount; i++) {
            if (components[i] == component) {
                return i;
            }
        }
        return -1;
    }

    public void remove(SignComponent component) {
        int index = getComponentIndex(component);
        if (index >= 0) {
            componentCount--;
            if (index == componentCount) {
                components[index] = null;
            } else {
    	        System.arraycopy(components,index+1,components,index,componentCount-index);
            }
        }
    }
    
    public void toFront(SignComponent component) {
        remove(component);
        add(component);
    }

/*** THIS LINE IS NECESSARY RIGHT NOW TO OVERCOME WIERD JVM OR COMPILER
 *** BUG WHERE getBounds RETURNS THE WRONG SIZE UNLESS THIS IS HERE ***/
static SignCharacter temp = new SignCharacter((byte)0,(byte)0,(byte)0);
/****/
    public Rectangle getBounds() {
        Rectangle union = null;
        for (int i = 0; i < componentCount; i++) {
            Rectangle compBounds = 
                new Rectangle(components[i].x,
                              components[i].y,
                              components[i].getWidth(),
                              components[i].getHeight());
            if (union == null) {
                union = compBounds;
            } else {
                union = union.union(compBounds);
            }
        }
        if (union == null) {
            union = new Rectangle(0,0,0,0);
        }
        return union;
    }

    public Point getOffset() {
        Point offset = new Point(0,0);
        for (int i = 0; i < componentCount; i++) {
            if (i == 0) {
                offset.x = components[i].x;
                offset.y = components[i].y;
            } else {
                offset.x = Math.min(offset.x,components[i].x);
                offset.y = Math.min(offset.y,components[i].y);
            }
        }
        return offset;
    }

    public void deselectAllComponents() {
        for (int i = 0; i < componentCount; i++) {
            components[i].setSelected(false);
        }
    }

    public int getSelectedComponentCount() {
        int count = 0;
        for (int i = 0; i < componentCount; i++) {
            if (components[i].isSelected()) {
                count++;
            }
        }
        return count;
    }

    public SignComponent[] getSelectedComponents() {
        SignComponent[] selected = new SignComponent[getSelectedComponentCount()];
        int j = 0;
        for (int i = 0; i < componentCount; i++) {
            if (components[i].isSelected()) {
                selected[j++] = components[i];
            }
        }
        return selected;
    }

    public SignComponent getComponentAt(int x, int y) {
        for (int i = componentCount-1; i >= 0; i--) {
            if (x >= components[i].x && y >= components[i].y &&
                x < (components[i].x + components[i].getWidth()) &&
                y < (components[i].y + components[i].getHeight())) {
                return components[i];
            }
        }
        return null;
    }


    public void paint(GadgetGraphics g, int x, int y) {
        paint(g,x,y,0,0,null,null,null);
    }

static int paints = 0;
static int painttotal = 0;
static int paintavg = 0;
static int paintmax = 0;
static int paintmin = 10000000;

    public void paint(GadgetGraphics g, int x, int y, int offsetx, int offsety,
                     Bitmap bitmap, int[] pixels, PaintInfo paintInfo) {
long start = System.currentTimeMillis();
        SignFont signFont;
        Font font = g.getFont();
        if (font instanceof SignFont) {
            signFont = (SignFont)font;
        } else {
            font = signFont = SignFont.large;
            g.setFont(font);
        }
        //Dimension size = getPreferredSize();
        Color normColor = g.getColor();
        for (int i = 0; i < componentCount; i++) {
            if (components[i].isSelected()) {
                g.setColor(getSelectedColor());
            } else {
                g.setColor(normColor);
            }
            components[i].paint(g,x,y, offsetx, offsety,
                                bitmap, pixels, paintInfo);
        }
int dur = (int)(System.currentTimeMillis()-start);
paints++;
painttotal += dur;
paintavg = painttotal/paints;
paintmax = Math.max(paintmax,dur);
paintmin = Math.min(paintmin,dur);
        //return x+Math.max(nextSignPos,(size.width + 10));
    }

}
