package org.signwriting;

import com.dtai.gwt.GadgetGraphics;
import java.awt.Dimension;
import java.awt.Point;

public abstract class SignComponent implements Cloneable, java.io.Serializable {
    
    static final long serialVersionUID = -6238687209950664445L;
    
    protected int x;
    protected int y;
    private transient int width;
    private transient int height;
    
    private transient boolean selected = false; // GET RID OF THIS TOO???!!!

    protected SignComponent() {
    }

    protected SignComponent(SignComponent rhs) {
        x = rhs.x;
        y = rhs.y;
        width = rhs.width;
        height = rhs.height;
    }

    public abstract Object clone();
    
    public final int getWidth() {
        if (width == 0) {
            width = calcWidth();
        }
        return width;
    }
    
    public final int getHeight() {
        if (height == 0) {
            height = calcHeight();
        }
        return height;
    }
    
    protected abstract int calcWidth();
    protected abstract int calcHeight();
    
    public void resetSize() {
        width = 0;
        height = 0;
    }

    public boolean equals(Object obj) {
        if (obj.getClass() != getClass()) {
            return false;
        }
        SignComponent rhs = (SignComponent)obj;
        if (rhs.x != x ||
            rhs.y != y ||
            rhs.width != getWidth() ||
            rhs.height != getHeight()) {
            return false;
        }
        return true;
    }

    public Dimension getSize() {
        return new Dimension(getWidth(),getHeight());
    }
    
    public Point getLocation() {
        return new Point(x, y);
    }
    
    public void setLocation(int x, int y) {
        this.x = x;
        this.y = y;
    }

    public void setSelected(boolean selected) {
        this.selected = selected;
    }

    public boolean isSelected() {
        return selected;
    }

    public void paint(GadgetGraphics g, int x, int y) {
        paint(g,x,y,0,0,null,null,null);
    }

    public abstract void paint(GadgetGraphics g, int x, int y,
                               int offsetx, int offsety,
                               Bitmap bitmap, int[] pixels,
                               PaintInfo paintInfo);
}
