/****************************************************************
 **
 **  $Id: SignFont.java,v 1.16 1998/09/14 18:16:27 ccp Exp $
 **
 **  $Source: /cvs/classes/signwriting/SignFont.java,v $
 **
 ****************************************************************/

package org.signwriting;

import com.dtai.gwt.GadgetGraphics;
import java.awt.Dimension;
import com.dtai.gwt.SpecialFont;
import com.dtai.gwt.SpecialFontMetrics;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.Image;
import java.awt.image.MemoryImageSource;
import java.util.Hashtable;

/**
 * SignFont
 * @version 1.1
 * @author DTAI, Incorporated
 */
public class SignFont extends SpecialFont {
    
    private static final int SMALL_MARGIN = 10; // GAP TO RIGHT SIDE
    private static final int LARGE_MARGIN = SMALL_MARGIN*2; // GAP TO RIGHT SIDE

    public static final char FONT_INDEX_START = 256;
    public static final char LARGE_FONT_SIZE = 18;
    public static final char SMALL_FONT_SIZE = 9;

    public static SignFont large = new SignFont();
    public static SignFont small = new SignFont(true);
    
//    int margin;
    int horizontalMargin;
    int verticalMargin;

    private static Hashtable signHash = new Hashtable();
    
    // RWK: I had to put the initilization in a member function
    // because if it is in the constructor or static initializer,
    // serialver fails!
    private static Sign[] signArray;// = new Sign[1024];
    
    private static char nextFontIndex = FONT_INDEX_START;
    private static int maxHeight;

    private SignFontMetrics metrics;
    private int descent;

    private SignFont() {
        this(false);
    }
    
    public boolean charIsWord(char ch) {
        //Sign sign = signArray[ch-FONT_INDEX_START];
        //return ((sign != null) && sign.isWord());
        return (signArray != null && signArray[ch] != null && signArray[ch].isWord());
    }

    private SignFont(boolean thumbNail) {
        super("Courier",Font.PLAIN,(thumbNail?SMALL_FONT_SIZE:LARGE_FONT_SIZE));
        
        if (thumbNail) {
            horizontalMargin = SMALL_MARGIN;
            verticalMargin = SMALL_MARGIN;
        } else {
            horizontalMargin = LARGE_MARGIN;
            verticalMargin = LARGE_MARGIN;
        }
        metrics = new SignFontMetrics(this,thumbNail);
        descent = metrics.getDescent();
        maxHeight = super.getSize() - descent;
        if (thumbNail) {
            maxHeight *= 2;
        }
    }

    public SpecialFontMetrics getMetrics() {
        return metrics;
    }

    public static void add(Sign sign) {
        synchronized(signHash) {
            
            // RWK: I had to put this here because if it is in the
            // constructor or static initializer, serialver fails!
            if (signArray == null) {
                signArray = new Sign[1024];
            }
            
            Object match = signHash.get(sign);
            if (match != null) {
                sign.setFontIndex((char)((Integer)match).intValue());
                sign.removeAllImages();
            } else {
                if (nextFontIndex == 0xffff) {
                    throw new IndexOutOfBoundsException("Too many signs!");
                }
                sign.setFontIndex(nextFontIndex++);
                sign.removeAllImages();
                int idx = sign.getFontIndex()-FONT_INDEX_START;
                if (idx >= signArray.length) {
                    Sign[] newSignArray = new Sign[signArray.length*2];
                    System.arraycopy(newSignArray,0,signArray,0,signArray.length);
                    signArray = newSignArray;
                }
                signArray[idx] = (Sign)sign.clone();
                signHash.put(signArray[idx],new Integer(sign.getFontIndex()));
                maxHeight = Math.max(maxHeight,sign.getBounds().height);
            }
        }
    }

    public static Sign getSign(char fontIndex) {
        int idx = fontIndex-FONT_INDEX_START;
        if ((signArray != null) &&
            (idx >= 0) &&
            (idx < signArray.length)) {
            return signArray[idx];
        }
        return null;
    }

    public int getSize() {
        if (this == small) {
            return maxHeight / 2;
        } else {
            return maxHeight;
        }
    }

    public void drawString(GadgetGraphics g, String str, int x, int y, int top, int bottom) {
  //      y += verticalMargin; // not sure why I have to do this
        int length = str.length();
        int maxHeight = getSize();
        boolean prevCharWasSignWord = false;
        PaintInfo paintInfo = new PaintInfo();
        paintInfo.color = 0xff000000 | g.getColor().getRGB();
        for (int i = 0; i < length; i++) {
            char ch = str.charAt(i);
            if (ch >= FONT_INDEX_START) {
                Sign sign = getSign(ch);
                if (prevCharWasSignWord) {
                    g.drawLine(x/*+(horizontalMargin/2)*/,y+1,x/*+(horizontalMargin/2)*/,y-1);
                    x += horizontalMargin;
                }
                if (sign.isWord()) {
                    prevCharWasSignWord = true;
                } else {
                    prevCharWasSignWord = false;
                }
                if (sign != null) {
                    Rectangle bounds = sign.getBounds();
                    if (this == small) {
                        bounds.x /= 2;
                        bounds.y /= 2;
                        bounds.width++;
                        bounds.height++;
                        bounds.width /= 2;
                        bounds.height /= 2;
                    }
                    int paintx = x;
                    int painty = y-bounds.height+1;
                    if (sign.isWord() && (bottom > top)) {
                        painty -= ((bottom-top) - bounds.height)/2;
                    }
                    Image image = sign.getImage((this==small),g.getColor());
                    if (image == null && sign.isImageOK()) {
// we could change this to pass in the pixel array instead, 
// but we need to calculate the height and width first.
                        //Bitmap bitmap = new Bitmap();
                        int[] pixels = new int[bounds.width*bounds.height];
                        paintInfo.valid = true;
                        paintInfo.width = bounds.width;
                        sign.paint(g, paintx, painty, -bounds.x, -bounds.y,
                                   /*bitmap*/null,pixels,paintInfo);
                        if (!paintInfo.valid) {
                            sign.setImageOK(false);
                        } else {
                            /*
                            int width = bitmap.getWidth();
                            int height = bitmap.getHeight();
                            int[] pixels = new int[width*height];
                            int colorbits = g.getColor().getRGB();
                            for (int col = 0; col < width; col++) {
                                for (int row = 0; row < height; row++) {
                                    if (bitmap.getBit(col,row)) {
                                        pixels[row*width+col] = 0xff000000 | colorbits;
                                    }
                                }
                            }
                            */
                            image = Toolkit.getDefaultToolkit().createImage(
                                new MemoryImageSource (bounds.width, bounds.height,
                                                       pixels, 0, bounds.width));
                            sign.addImage(image, (this==small), g.getColor());
                        }
                    }
                    if (!sign.isImageOK()) {
                        sign.paint(g,paintx,painty, -bounds.x, -bounds.y,
                                   null, null, null);
                    } else {
                        g.drawImage(image, paintx, painty, g.getGadget());
                    }
                    x += bounds.width + 1;
                    if (sign.isWord()) {
                        x += horizontalMargin;
                    }
                }
            } else {
                prevCharWasSignWord = false;
                char charArray[] = {ch};
                int charWidth = metrics.charsWidth(charArray,0,1);
                Graphics awtG = g.getOrigGraphics();
                Point offset = g.getOffset();
                awtG.drawChars(charArray, 0, 1, x+offset.x, y+offset.y+1);
                x += charWidth;
            }
        }
    }
}
